@@ -1,10 +1,10 @@
-/* aewm - Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>.
+/* aewm - Copyright 1998-2007 Decklin Foster <decklin@red-bean.com>.
  * This program is free software; please see LICENSE for details. */
 
 #ifndef AEWM_H
 #define AEWM_H
 
-#define VERSION "1.3.10"
+#define VERSION "1.3.11"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -65,13 +65,13 @@
 #define MouseMask (ButtonMask|PointerMotionMask)
 
 #define BW(c) ((c)->decor ? opt_bw : 0)
+#define GRAV(c) ((c->size.flags & PWinGravity) ? c->size.win_gravity : \
+    NorthWestGravity)
 #define CAN_PLACE_SELF(t) ((t) == net_wm_type_dock || \
     (t) == net_wm_type_menu || (t) == net_wm_type_splash || \
     (t) == net_wm_type_desk)
 #define HAS_DECOR(t) (!CAN_PLACE_SELF(t))
 #define IS_ON_CUR_DESK(c) IS_ON_DESK((c)->desk, cur_desk)
-#define GRAB(w, mask, curs) (XGrabPointer(dpy, w, False, mask, \
-    GrabModeAsync, GrabModeAsync, None, curs, CurrentTime) == GrabSuccess)
 
 #ifdef DEBUG
 #define SHOW_EV(name, memb) \
@@ -93,11 +93,11 @@ struct client {
     client_t *next;
     char *name;
     Window win, frame, trans;
+    geom_t geom, save;
 #ifdef XFT
     XftDraw *xftdraw;
 #endif
-    XSizeHints *size;
-    geom_t geom, save;
+    XSizeHints size;
     Colormap cmap;
     int ignore_unmap;
     unsigned long desk;
@@ -107,6 +107,7 @@ struct client {
     Bool shaded;
     Bool zoomed;
     Bool decor;
+    int old_bw;
 };
 
 typedef void sweep_func(client_t *, geom_t, int, int, int, int, strut_t *);
