@@ -1,4 +1,4 @@
-/* aewm - Copyright 1998-2006 Decklin Foster <decklin@red-bean.com>.
+/* aewm - Copyright 1998-2007 Decklin Foster <decklin@red-bean.com>.
  * This program is free software; please see LICENSE for details. */
 
 #include <stdlib.h>
@@ -35,18 +35,19 @@ client_t *new_client(Window w)
     c->next = head;
     head = c;
 
+    c->name = get_wm_name(w);
     c->win = w;
     c->frame = None;
-    c->shaded = 0;
-    c->zoomed = 0;
+    c->size.flags = 0;
+    c->ignore_unmap = 0;
 #ifdef SHAPE
     c->shaped = 0;
 #endif
-    c->ignore_unmap = 0;
+    c->shaded = 0;
+    c->zoomed = 0;
+    c->decor = 1;
 
-    c->name = get_wm_name(w);
-    c->size = XAllocSizeHints();
-    XGetWMNormalHints(dpy, c->win, c->size, &supplied);
+    XGetWMNormalHints(dpy, c->win, &c->size, &supplied);
     XGetTransientForHint(dpy, c->win, &c->trans);
 
     XGetWindowAttributes(dpy, c->win, &attr);
@@ -55,6 +56,7 @@ client_t *new_client(Window w)
     c->geom.w = attr.width;
     c->geom.h = attr.height;
     c->cmap = attr.colormap;
+    c->old_bw = attr.border_width;
 
 #ifdef DEBUG
     dump_name(c, "creating", 'w');
@@ -67,16 +69,18 @@ client_t *new_client(Window w)
 
     if (get_atoms(c->win, net_wm_wintype, XA_ATOM, 0, &win_type, 1, NULL))
         c->decor = HAS_DECOR(win_type);
-    else
-        c->decor = 1;
 
     if (get_atoms(c->win, net_wm_desk, XA_CARDINAL, 0, &c->desk, 1, NULL)) {
+        if (c->desk == -1) c->desk = DESK_ALL; /* FIXME */
         if (c->desk >= ndesks && c->desk != DESK_ALL)
-            c->desk = ndesks - 1;
+            c->desk = cur_desk;
     } else {
         set_atoms(c->win, net_wm_desk, XA_CARDINAL, &cur_desk, 1);
         c->desk = cur_desk;
     }
+#ifdef DEBUG
+    dump_info(c);
+#endif
 
     check_states(c);
 
@@ -165,7 +169,7 @@ static void do_map(client_t *c, int do_raise)
 {
     if (IS_ON_CUR_DESK(c)) {
         XMapWindow(dpy, c->win);
-        if (do_raise) { 
+        if (do_raise) {
             XMapRaised(dpy, c->frame);
         } else {
             XLowerWindow(dpy, c->frame);
@@ -191,7 +195,6 @@ static int init_geom(client_t *c, strut_t *s)
     int wmax = screen_x - s->left - s->right;
     int hmax = screen_y - s->top - s->bottom;
     int mouse_x, mouse_y;
-    geom_t f = frame_geom(c);
 
     /* We decide the geometry for these types of windows, so we can just
      * ignore everything and return right away. If c->zoomed is set, that
@@ -200,8 +203,8 @@ static int init_geom(client_t *c, strut_t *s)
         return 1;
     if (get_atoms(c->win, net_wm_state, XA_ATOM, 0, &state, 1, NULL) &&
             state == net_wm_state_fs) {
-        c->geom.x = 0 - opt_bw;
-        c->geom.y = 0 - opt_bw;
+        c->geom.x = 0;
+        c->geom.y = 0;
         c->geom.w = screen_x;
         c->geom.h = screen_y;
         return 1;
@@ -210,13 +213,13 @@ static int init_geom(client_t *c, strut_t *s)
     /* Here, we merely set the values; they're in the same place regardless
      * of whether the user or the program specified them. We'll distinguish
      * between the two cases later, if we need to. */
-    if (c->size->flags & (USSize|PSize)) {
-        if (c->size->width > 0) c->geom.w = c->size->width;
-        if (c->size->height > 0) c->geom.h = c->size->height;
+    if (c->size.flags & (USSize|PSize)) {
+        if (c->size.width > 0) c->geom.w = c->size.width;
+        if (c->size.height > 0) c->geom.h = c->size.height;
     }
-    if (c->size->flags & (USPosition|PPosition)) {
-        if (c->size->x > 0) c->geom.x = c->size->x;
-        if (c->size->y > 0) c->geom.y = c->size->y;
+    if (c->size.flags & (USPosition|PPosition)) {
+        if (c->size.x > 0) c->geom.x = c->size.x;
+        if (c->size.y > 0) c->geom.y = c->size.y;
     }
 
     /* Several types of windows can put themselves wherever they want, but we
@@ -237,16 +240,20 @@ static int init_geom(client_t *c, strut_t *s)
      * for transients, because they might be a panel-type client popping up a
      * notification window over themselves. */
     if (!c->trans) {
-        if (f.x + f.w > screen_x - s->right) f.x = s->right - f.w;
-        if (f.y + f.h > screen_y - s->bottom) f.y = s->bottom - f.h;
-        if (f.x < s->left || f.w > wmax) f.x = s->left;
-        if (f.y < s->top || f.h > hmax) f.y = s->top;
+        if (c->geom.x + c->geom.w > screen_x - s->right)
+            c->geom.x = screen_x - s->right - c->geom.w;
+        if (c->geom.y + c->geom.h > screen_y - s->bottom)
+            c->geom.y = screen_y - s->bottom - c->geom.h;
+        if (c->geom.x < s->left || c->geom.w > wmax)
+            c->geom.x = s->left;
+        if (c->geom.y < s->top || c->geom.h > hmax)
+            c->geom.y = s->top;
     }
 
     /* Finally, we decide if we were ultimately satisfied with the position
      * given, or if we had to make something up, so that the caller can
      * consider using some other method. */
-    return c->trans || c->size->flags & USPosition;
+    return c->trans || c->size.flags & USPosition;
 }
 
 /* The frame window is not created until we actually do the reparenting here,
@@ -304,7 +311,7 @@ int frame_height(client_t *c)
 #else
         font_height = font->ascent + font->descent;
 #endif
-        return c->trans ? font_height / 2 : font_height + 2*opt_pad + BW(c);
+        return (c->trans ? 0 : font_height) + 2 * opt_pad + BW(c);
     } else {
         return 0;
     }
@@ -375,7 +382,7 @@ void redraw_frame(client_t *c)
 #ifdef XFT
 #ifdef X_HAVE_UTF8_STRING
             XftDrawStringUtf8(c->xftdraw, &xft_fg,
-                xftfont, opt_pad, opt_pad + xftfont->ascent,
+                xftfont, opt_pad + xftfont->descent / 2, opt_pad + xftfont->ascent,
                 (unsigned char *)c->name, strlen(c->name));
 #else
             XftDrawString8(c->xftdraw, &xft_fg,
@@ -409,23 +416,47 @@ void redraw_frame(client_t *c)
 geom_t frame_geom(client_t *c)
 {
     geom_t f = c->geom;
-    int gravity = (c->size->flags & PWinGravity) ?
-        c->size->win_gravity : NorthWestGravity;
 
-    if (c->shaded)
-        f.h = frame_height(c) - BW(c);
-    else
-        f.h += frame_height(c);
+    /* everything else is the same as c->geom was */
+    f.h = frame_height(c) + (c->shaded ? -BW(c) : c->geom.h);
 
-    switch (gravity) {
-        case CenterGravity:
-            f.y -= frame_height(c)/2;
+    /* X, in its perpetual helpfulness, always does native borders NorthWest
+     * style. This, as usual, ruins everything. So we compensate. */
+    switch (GRAV(c)) {
+        case NorthWestGravity:
+            break;
+        case NorthGravity:
+            f.x -= BW(c);
+            break;
+        case NorthEastGravity:
+            f.x -= 2 * BW(c);
+            break;
+        case EastGravity:
+            f.x -= 2 * BW(c);
+            f.y -= frame_height(c) / 2 + BW(c);
+            break;
+        case SouthEastGravity:
+            f.x -= 2 * BW(c);
+            f.y -= frame_height(c) + 2 * BW(c);
+            break;
+        case SouthGravity:
+            f.x -= BW(c);
+            f.y -= frame_height(c) + 2 * BW(c);
             break;
         case SouthWestGravity:
-        case SouthEastGravity:
-        case SouthGravity:
+            f.y -= frame_height(c) + 2 * BW(c);
+            break;
+        case WestGravity:
+            f.y -= frame_height(c) / 2 + BW(c);
+            break;
         case StaticGravity:
-            f.y -= frame_height(c); break;
+            f.y -= frame_height(c) + BW(c);
+            f.x -= BW(c);
+            break;
+        case CenterGravity:
+            f.x -= BW(c);
+            f.y -= frame_height(c) / 2 + BW(c);
+            break;
     }
 
     return f;
@@ -532,8 +563,7 @@ void del_client(client_t *c, int mode)
     remove_atom(root, net_client_list, XA_WINDOW, c->win);
     remove_atom(root, net_client_stack, XA_WINDOW, c->win);
 
-    if (c->decor)
-        XSetWindowBorderWidth(dpy, c->win, 1);
+    XSetWindowBorderWidth(dpy, c->win, c->old_bw);
 #ifdef XFT
     if (c->xftdraw)
         XftDrawDestroy(c->xftdraw);
@@ -550,7 +580,6 @@ void del_client(client_t *c, int mode)
             p->next = c->next;
 
     if (c->name) XFree(c->name);
-    if (c->size) XFree(c->size);
     free(c);
 
     XSync(dpy, False);
